@@ -17,13 +17,14 @@
 # SOFTWARE.
 """
 This module implements the adversarial patch attack `AdversarialPatch`. This attack generates an adversarial patch that
-can be printed into the physical world with a common printer. The patch can be used to fool image and video classifiers.
+can be printed into the physical world with a common printer. The patch can be used to fool image and video estimators.
 
 | Paper link: https://arxiv.org/abs/1712.09665
 """
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
+import math
 from typing import Optional, Tuple, Union, TYPE_CHECKING
 
 import numpy as np
@@ -32,7 +33,6 @@ from tqdm.auto import trange
 from art.attacks.attack import EvasionAttack
 from art.attacks.evasion.adversarial_patch.utils import insert_transformed_patch
 from art.estimators.estimator import BaseEstimator, NeuralNetworkMixin
-from art.estimators.classification.classifier import ClassifierMixin
 from art.utils import check_and_transform_label_format, is_probability, to_categorical
 from art.summary_writer import SummaryWriter
 
@@ -61,15 +61,17 @@ class AdversarialPatchPyTorch(EvasionAttack):
         "max_iter",
         "batch_size",
         "patch_shape",
+        "optimizer",
+        "targeted",
         "summary_writer",
         "verbose",
     ]
 
-    _estimator_requirements = (BaseEstimator, NeuralNetworkMixin, ClassifierMixin)
+    _estimator_requirements = (BaseEstimator, NeuralNetworkMixin)
 
     def __init__(
         self,
-        classifier: "CLASSIFIER_NEURALNETWORK_TYPE",
+        estimator: "CLASSIFIER_NEURALNETWORK_TYPE",
         rotation_max: float = 22.5,
         scale_min: float = 0.1,
         scale_max: float = 1.0,
@@ -77,15 +79,18 @@ class AdversarialPatchPyTorch(EvasionAttack):
         learning_rate: float = 5.0,
         max_iter: int = 500,
         batch_size: int = 16,
-        patch_shape: Optional[Tuple[int, int, int]] = None,
+        patch_shape: Tuple[int, int, int] = (3, 224, 224),
+        patch_location: Optional[Tuple[int, int]] = None,
         patch_type: str = "circle",
+        optimizer: str = "Adam",
+        targeted: bool = True,
         summary_writer: Union[str, bool, SummaryWriter] = False,
         verbose: bool = True,
     ):
         """
         Create an instance of the :class:`.AdversarialPatchPyTorch`.
 
-        :param classifier: A trained classifier.
+        :param estimator: A trained estimator.
         :param rotation_max: The maximum rotation applied to random patches. The value is expected to be in the
                range `[0, 180]`.
         :param scale_min: The minimum scaling applied to random patches. The value should be in the range `[0, 1]`,
@@ -94,11 +99,16 @@ class AdversarialPatchPyTorch(EvasionAttack):
                larger than `scale_min`.
         :param distortion_scale_max: The maximum distortion scale for perspective transformation in range `[0, 1]`. If
                distortion_scale_max=0.0 the perspective transformation sampling will be disabled.
-        :param learning_rate: The learning rate of the optimization.
+        :param learning_rate: The learning rate of the optimization. For `optimizer="pgd"` the learning rate gets
+                              multiplied with the sign of the loss gradients.
         :param max_iter: The number of optimization steps.
         :param batch_size: The size of the training batch.
         :param patch_shape: The shape of the adversarial patch as a tuple of shape CHW (nb_channels, height, width).
+        :param patch_location: The location of the adversarial patch as a tuple of shape (upper left x, upper left y).
         :param patch_type: The patch type, either circle or square.
+        :param optimizer: The optimization algorithm. Supported values: "Adam", and "pgd". "pgd" corresponds to
+                          projected gradient descent in L-Inf norm.
+        :param targeted: Indicates whether the attack is targeted (True) or untargeted (False).
         :param summary_writer: Activate summary writer for TensorBoard.
                                Default is `False` and deactivated summary writer.
                                If `True` save runs/CURRENT_DATETIME_HOSTNAME in current directory.
@@ -111,14 +121,14 @@ class AdversarialPatchPyTorch(EvasionAttack):
         import torch  # lgtm [py/repeated-import]
         import torchvision
 
-        torch_version = list(map(int, torch.__version__.lower().split("+")[0].split(".")))
-        torchvision_version = list(map(int, torchvision.__version__.lower().split("+")[0].split(".")))
+        torch_version = list(map(int, torch.__version__.lower().split("+", maxsplit=1)[0].split(".")))
+        torchvision_version = list(map(int, torchvision.__version__.lower().split("+", maxsplit=1)[0].split(".")))
         assert torch_version[0] >= 1 and torch_version[1] >= 7, "AdversarialPatchPyTorch requires torch>=1.7.0"
         assert (
             torchvision_version[0] >= 0 and torchvision_version[1] >= 8
         ), "AdversarialPatchPyTorch requires torchvision>=0.8.0"
 
-        super().__init__(estimator=classifier, summary_writer=summary_writer)
+        super().__init__(estimator=estimator, summary_writer=summary_writer)
         self.rotation_max = rotation_max
         self.scale_min = scale_min
         self.scale_max = scale_max
@@ -126,19 +136,15 @@ class AdversarialPatchPyTorch(EvasionAttack):
         self.learning_rate = learning_rate
         self.max_iter = max_iter
         self.batch_size = batch_size
-        if patch_shape is None:
-            self.patch_shape = self.estimator.input_shape
-        else:
-            self.patch_shape = patch_shape
+        self.patch_shape = patch_shape
+        self.patch_location = patch_location
         self.patch_type = patch_type
 
-        self.image_shape = classifier.input_shape
+        self.image_shape = estimator.input_shape
+        self.targeted = targeted
         self.verbose = verbose
         self._check_params()
 
-        if not self.estimator.channels_first:  # pragma: no cover
-            raise ValueError("Input shape has to be wither NCHW or NFCHW.")
-
         self.i_h_patch = 1
         self.i_w_patch = 2
 
@@ -169,26 +175,39 @@ class AdversarialPatchPyTorch(EvasionAttack):
         self._initial_value = np.ones(self.patch_shape) * mean_value
         self._patch = torch.tensor(self._initial_value, requires_grad=True, device=self.estimator.device)
 
-        self._optimizer = torch.optim.Adam([self._patch], lr=self.learning_rate)
+        self._optimizer_string = optimizer
+        if self._optimizer_string == "Adam":
+            self._optimizer = torch.optim.Adam([self._patch], lr=self.learning_rate)
 
     def _train_step(
         self, images: "torch.Tensor", target: "torch.Tensor", mask: Optional["torch.Tensor"] = None
     ) -> "torch.Tensor":
         import torch  # lgtm [py/repeated-import]
 
-        self._optimizer.zero_grad()
+        self.estimator.model.zero_grad()
         loss = self._loss(images, target, mask)
         loss.backward(retain_graph=True)
-        self._optimizer.step()
 
-        with torch.no_grad():
-            self._patch[:] = torch.clamp(
-                self._patch, min=self.estimator.clip_values[0], max=self.estimator.clip_values[1]
-            )
+        if self._optimizer_string == "pgd":
+            gradients = self._patch.grad.sign() * self.learning_rate
+
+            with torch.no_grad():
+                self._patch[:] = torch.clamp(
+                    self._patch + gradients, min=self.estimator.clip_values[0], max=self.estimator.clip_values[1]
+                )
+        else:
+            self._optimizer.step()
+
+            with torch.no_grad():
+                self._patch[:] = torch.clamp(
+                    self._patch, min=self.estimator.clip_values[0], max=self.estimator.clip_values[1]
+                )
 
         return loss
 
-    def _predictions(self, images: "torch.Tensor", mask: Optional["torch.Tensor"]) -> "torch.Tensor":
+    def _predictions(
+        self, images: "torch.Tensor", mask: Optional["torch.Tensor"], target: "torch.Tensor"
+    ) -> Tuple["torch.Tensor", "torch.Tensor"]:
         import torch  # lgtm [py/repeated-import]
 
         patched_input = self._random_overlay(images, self._patch, mask=mask)
@@ -198,23 +217,37 @@ class AdversarialPatchPyTorch(EvasionAttack):
             max=self.estimator.clip_values[1],
         )
 
-        predictions = self.estimator._predict_framework(patched_input)  # pylint: disable=W0212
+        predictions, target = self.estimator._predict_framework(patched_input, target)  # pylint: disable=W0212
 
-        return predictions
+        return predictions, target
 
     def _loss(self, images: "torch.Tensor", target: "torch.Tensor", mask: Optional["torch.Tensor"]) -> "torch.Tensor":
         import torch  # lgtm [py/repeated-import]
 
-        predictions = self._predictions(images, mask)
+        if isinstance(target, torch.Tensor):
+
+            predictions, target = self._predictions(images, mask, target)
+
+            if self.use_logits:
+                loss = torch.nn.functional.cross_entropy(
+                    input=predictions, target=torch.argmax(target, dim=1), reduction="mean"
+                )
+            else:
+                loss = torch.nn.functional.nll_loss(
+                    input=predictions, target=torch.argmax(target, dim=1), reduction="mean"
+                )
 
-        if self.use_logits:
-            loss = torch.nn.functional.cross_entropy(
-                input=predictions, target=torch.argmax(target, dim=1), reduction="mean"
-            )
         else:
-            loss = torch.nn.functional.nll_loss(input=predictions, target=torch.argmax(target, dim=1), reduction="mean")
+            patched_input = self._random_overlay(images, self._patch, mask=mask)
+            patched_input = torch.clamp(
+                patched_input,
+                min=self.estimator.clip_values[0],
+                max=self.estimator.clip_values[1],
+            )
 
-        if not self.targeted:
+            loss = self.estimator.compute_loss(x=patched_input, y=target)
+
+        if (not self.targeted and self._optimizer_string != "pgd") or self.targeted and self._optimizer_string == "pgd":
             loss = -loss
 
         return loss
@@ -252,11 +285,17 @@ class AdversarialPatchPyTorch(EvasionAttack):
         import torch  # lgtm [py/repeated-import]
         import torchvision
 
+        # Ensure channels-first
+        if not self.estimator.channels_first:
+            images = torch.permute(images, (0, 3, 1, 2))
+
         nb_samples = images.shape[0]
 
         image_mask = self._get_circular_patch_mask(nb_samples=nb_samples)
         image_mask = image_mask.float()
 
+        self.image_shape = images.shape[1:]
+
         smallest_image_edge = np.minimum(self.image_shape[self.i_h], self.image_shape[self.i_w])
 
         image_mask = torchvision.transforms.functional.resize(
@@ -273,7 +312,7 @@ class AdversarialPatchPyTorch(EvasionAttack):
 
         image_mask = torchvision.transforms.functional.pad(
             img=image_mask,
-            padding=[pad_h_before, pad_w_before, pad_h_after, pad_w_after],
+            padding=[pad_w_before, pad_h_before, pad_w_after, pad_h_after],
             fill=0,
             padding_mode="constant",
         )
@@ -295,7 +334,7 @@ class AdversarialPatchPyTorch(EvasionAttack):
 
         padded_patch = torchvision.transforms.functional.pad(
             img=padded_patch,
-            padding=[pad_h_before, pad_w_before, pad_h_after, pad_w_after],
+            padding=[pad_w_before, pad_h_before, pad_w_after, pad_h_after],
             fill=0,
             padding_mode="constant",
         )
@@ -306,24 +345,33 @@ class AdversarialPatchPyTorch(EvasionAttack):
 
         padded_patch = padded_patch.float()
 
-        image_mask_list = list()
-        padded_patch_list = list()
+        image_mask_list = []
+        padded_patch_list = []
 
         for i_sample in range(nb_samples):
-            if scale is None:
-                im_scale = np.random.uniform(low=self.scale_min, high=self.scale_max)
+            if self.patch_location is None:
+                if scale is None:
+                    im_scale = np.random.uniform(low=self.scale_min, high=self.scale_max)
+                else:
+                    im_scale = scale
             else:
-                im_scale = scale
+                im_scale = self.patch_shape[self.i_h] / smallest_image_edge
 
             if mask is None:
-                padding_after_scaling_h = (
-                    self.image_shape[self.i_h] - im_scale * padded_patch.shape[self.i_h + 1]
-                ) / 2.0
-                padding_after_scaling_w = (
-                    self.image_shape[self.i_w] - im_scale * padded_patch.shape[self.i_w + 1]
-                ) / 2.0
-                x_shift = np.random.uniform(-padding_after_scaling_w, padding_after_scaling_w)
-                y_shift = np.random.uniform(-padding_after_scaling_h, padding_after_scaling_h)
+                if self.patch_location is None:
+                    padding_after_scaling_h = (
+                        self.image_shape[self.i_h] - im_scale * padded_patch.shape[self.i_h + 1]
+                    ) / 2.0
+                    padding_after_scaling_w = (
+                        self.image_shape[self.i_w] - im_scale * padded_patch.shape[self.i_w + 1]
+                    ) / 2.0
+                    x_shift = np.random.uniform(-padding_after_scaling_w, padding_after_scaling_w)
+                    y_shift = np.random.uniform(-padding_after_scaling_h, padding_after_scaling_h)
+                else:
+                    padding_h = int(math.floor(self.image_shape[self.i_h] - self.patch_shape[self.i_h]) / 2.0)
+                    padding_w = int(math.floor(self.image_shape[self.i_w] - self.patch_shape[self.i_w]) / 2.0)
+                    x_shift = -padding_w + self.patch_location[0]
+                    y_shift = -padding_h + self.patch_location[1]
             else:
                 mask_2d = mask[i_sample, :, :]
 
@@ -413,9 +461,16 @@ class AdversarialPatchPyTorch(EvasionAttack):
             torch.from_numpy(np.ones(shape=image_mask.shape, dtype=np.float32)).to(self.estimator.device) - image_mask
         )
 
-        return images * inverted_mask + padded_patch * image_mask
+        patched_images = images * inverted_mask + padded_patch * image_mask
 
-    def generate(self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs) -> Tuple[np.ndarray, np.ndarray]:
+        if not self.estimator.channels_first:
+            patched_images = torch.permute(patched_images, (0, 2, 3, 1))
+
+        return patched_images
+
+    def generate(  # type: ignore
+        self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs
+    ) -> Tuple[np.ndarray, np.ndarray]:
         """
         Generate an adversarial patch and return the patch and its mask in arrays.
 
@@ -435,39 +490,101 @@ class AdversarialPatchPyTorch(EvasionAttack):
             mask = mask.copy()
         mask = self._check_mask(mask=mask, x=x)
 
+        if self.patch_location is not None and mask is not None:
+            raise ValueError("Masks can only be used if the `patch_location` is `None`.")
+
         if y is None:  # pragma: no cover
             logger.info("Setting labels to estimator predictions and running untargeted attack because `y=None`.")
             y = to_categorical(np.argmax(self.estimator.predict(x=x), axis=1), nb_classes=self.estimator.nb_classes)
-            self.targeted = False
+
+        if hasattr(self.estimator, "nb_classes"):
+            y = check_and_transform_label_format(labels=y, nb_classes=self.estimator.nb_classes)
+
+            # check if logits or probabilities
+            y_pred = self.estimator.predict(x=x[[0]])
+
+            if is_probability(y_pred):
+                self.use_logits = False
+            else:
+                self.use_logits = True
+
+        if isinstance(y, np.ndarray):
+            x_tensor = torch.Tensor(x)
+            y_tensor = torch.Tensor(y)
+
+            if mask is None:
+                dataset = torch.utils.data.TensorDataset(x_tensor, y_tensor)
+                data_loader = torch.utils.data.DataLoader(
+                    dataset=dataset,
+                    batch_size=self.batch_size,
+                    shuffle=shuffle,
+                    drop_last=False,
+                )
+            else:
+                mask_tensor = torch.Tensor(mask)
+                dataset = torch.utils.data.TensorDataset(x_tensor, y_tensor, mask_tensor)
+                data_loader = torch.utils.data.DataLoader(
+                    dataset=dataset,
+                    batch_size=self.batch_size,
+                    shuffle=shuffle,
+                    drop_last=False,
+                )
         else:
-            self.targeted = True
 
-        y = check_and_transform_label_format(labels=y, nb_classes=self.estimator.nb_classes)
+            class ObjectDetectionDataset(torch.utils.data.Dataset):
+                """
+                Object detection dataset in PyTorch.
+                """
 
-        # check if logits or probabilities
-        y_pred = self.estimator.predict(x=x[[0]])
+                def __init__(self, x, y):
+                    self.x = x
+                    self.y = y
 
-        if is_probability(y_pred):
-            self.use_logits = False
-        else:
-            self.use_logits = True
+                def __len__(self):
+                    return self.x.shape[0]
 
-        x_tensor = torch.Tensor(x)
-        y_tensor = torch.Tensor(y)
+                def __getitem__(self, idx):
+                    img = torch.from_numpy(self.x[idx])
+
+                    target = {}
+                    target["boxes"] = torch.from_numpy(self.y[idx]["boxes"])
+                    target["labels"] = torch.from_numpy(self.y[idx]["labels"])
+                    target["scores"] = torch.from_numpy(self.y[idx]["scores"])
+
+                    return img, target
+
+            class ObjectDetectionDatasetMask(torch.utils.data.Dataset):
+                """
+                Object detection dataset in PyTorch.
+                """
+
+                def __init__(self, x, y, mask):
+                    self.x = x
+                    self.y = y
+                    self.mask = mask
+
+                def __len__(self):
+                    return self.x.shape[0]
+
+                def __getitem__(self, idx):
+                    img = torch.from_numpy(self.x[idx])
+
+                    target = {}
+                    target["boxes"] = torch.from_numpy(y[idx]["boxes"])
+                    target["labels"] = torch.from_numpy(y[idx]["labels"])
+                    target["scores"] = torch.from_numpy(y[idx]["scores"])
+                    mask_i = torch.from_numpy(self.mask[idx])
+
+                    return img, target, mask_i
+
+            dataset_object_detection: Union[ObjectDetectionDataset, ObjectDetectionDatasetMask]
+            if mask is None:
+                dataset_object_detection = ObjectDetectionDataset(x, y)
+            else:
+                dataset_object_detection = ObjectDetectionDatasetMask(x, y, mask)
 
-        if mask is None:
-            dataset = torch.utils.data.TensorDataset(x_tensor, y_tensor)
             data_loader = torch.utils.data.DataLoader(
-                dataset=dataset,
-                batch_size=self.batch_size,
-                shuffle=shuffle,
-                drop_last=False,
-            )
-        else:
-            mask_tensor = torch.Tensor(mask)
-            dataset = torch.utils.data.TensorDataset(x_tensor, y_tensor, mask_tensor)
-            data_loader = torch.utils.data.DataLoader(
-                dataset=dataset,
+                dataset=dataset_object_detection,
                 batch_size=self.batch_size,
                 shuffle=shuffle,
                 drop_last=False,
@@ -476,15 +593,24 @@ class AdversarialPatchPyTorch(EvasionAttack):
         for i_iter in trange(self.max_iter, desc="Adversarial Patch PyTorch", disable=not self.verbose):
             if mask is None:
                 for images, target in data_loader:
-                    images, target = images.to(self.estimator.device), target.to(self.estimator.device)
+                    images = images.to(self.estimator.device)
+                    if isinstance(target, torch.Tensor):
+                        target = target.to(self.estimator.device)
+                    else:
+                        target["boxes"] = target["boxes"].to(self.estimator.device)
+                        target["labels"] = target["labels"].to(self.estimator.device)
+                        target["scores"] = target["scores"].to(self.estimator.device)
                     _ = self._train_step(images=images, target=target, mask=None)
             else:
                 for images, target, mask_i in data_loader:
-                    images, target, mask_i = (
-                        images.to(self.estimator.device),
-                        target.to(self.estimator.device),
-                        mask_i.to(self.estimator.device),
-                    )
+                    images = images.to(self.estimator.device)
+                    if isinstance(target, torch.Tensor):
+                        target = target.to(self.estimator.device)
+                    else:
+                        target["boxes"] = target["boxes"].to(self.estimator.device)
+                        target["labels"] = target["labels"].to(self.estimator.device)
+                        target["scores"] = target["scores"].to(self.estimator.device)
+                    mask_i = mask_i.to(self.estimator.device)
                     _ = self._train_step(images=images, target=target, mask=mask_i)
 
             # Write summary
@@ -517,7 +643,7 @@ class AdversarialPatchPyTorch(EvasionAttack):
             self._get_circular_patch_mask(nb_samples=1).cpu().numpy()[0],
         )
 
-    def _check_mask(self, mask: np.ndarray, x: np.ndarray) -> np.ndarray:
+    def _check_mask(self, mask: Optional[np.ndarray], x: np.ndarray) -> Optional[np.ndarray]:
         if mask is not None and (  # pragma: no cover
             (mask.dtype != bool)
             or not (mask.shape[0] == 1 or mask.shape[0] == x.shape[0])
@@ -544,7 +670,7 @@ class AdversarialPatchPyTorch(EvasionAttack):
         A function to apply the learned adversarial patch to images or videos.
 
         :param x: Instances to apply randomly transformed patch.
-        :param scale: Scale of the applied patch in relation to the classifier input shape.
+        :param scale: Scale of the applied patch in relation to the estimator input shape.
         :param patch_external: External patch to apply to images `x`.
         :param mask: An boolean array of shape equal to the shape of a single samples (1, H, W) or the shape of `x`
                      (N, H, W) without their channel dimensions. Any features for which the mask is True can be the
@@ -556,9 +682,18 @@ class AdversarialPatchPyTorch(EvasionAttack):
         if mask is not None:
             mask = mask.copy()
         mask = self._check_mask(mask=mask, x=x)
-        patch = patch_external if patch_external is not None else self._patch
-        x = torch.Tensor(x)
-        return self._random_overlay(images=x, patch=patch, scale=scale, mask=mask).detach().cpu().numpy()
+        x_tensor = torch.Tensor(x)
+        mask_tensor = torch.Tensor(mask)
+        if isinstance(patch_external, np.ndarray):
+            patch_tensor = torch.Tensor(patch_external)
+        else:
+            patch_tensor = self._patch
+        return (
+            self._random_overlay(images=x_tensor, patch=patch_tensor, scale=scale, mask=mask_tensor)
+            .detach()
+            .cpu()
+            .numpy()
+        )
 
     def reset_patch(self, initial_patch_value: Optional[Union[float, np.ndarray]] = None) -> None:
         """
@@ -598,5 +733,17 @@ class AdversarialPatchPyTorch(EvasionAttack):
         if not isinstance(self.distortion_scale_max, (float, int)) or 1.0 <= self.distortion_scale_max < 0.0:
             raise ValueError("The maximum distortion scale has to be greater than or equal 0.0 or smaller than 1.0.")
 
+        if self.patch_location is not None and not (
+            isinstance(self.patch_location, tuple)
+            and len(self.patch_location) == 2
+            and isinstance(self.patch_location[0], int)
+            and self.patch_location[0] >= 0
+            and isinstance(self.patch_location[1], int)
+            and self.patch_location[1] >= 0
+        ):
+            raise ValueError(
+                "The patch location has to be either `None` or a tuple of two integers greater than or equal 0."
+            )
+
         if self.patch_type not in ["circle", "square"]:
             raise ValueError("The patch type has to be either `circle` or `square`.")
