@@ -546,9 +546,9 @@ class MalwareGDTensorFlow(EvasionAttack):
         if verify_input_data:
             if np.sum(y) != len(y):
                 raise ValueError(  # pragma: no cover
-                    "{} invalid samples found in batch which cannot support the assigned perturbation or "
-                    "are benign To filter for samples that can be processed use pull_out_valid_samples "
-                    "on the samples. Checking can be disabled by using verify_input_data".format(len(y) - np.sum(y))
+                    f"{len(y) - np.sum(y)} invalid samples found in batch which cannot support the assigned "
+                    f"perturbation or are benign To filter for samples that can be processed use "
+                    f"pull_out_valid_samples on the samples. Checking can be disabled by using verify_input_data"
                 )
 
         adv_x = self.initialise_sample(
@@ -595,7 +595,8 @@ class MalwareGDTensorFlow(EvasionAttack):
         :return data: A numpy array of the PE file
         :return size_of_original_file: Size of the PE file
         """
-        open_file = open(filepath, "rb").read()
+        with open(filepath, "rb") as file:
+            open_file = file.read()
         size_of_original_file = len(open_file)
 
         data = np.ones((maxlen,), dtype=np.uint16) * padding_char
@@ -638,7 +639,7 @@ class MalwareGDTensorFlow(EvasionAttack):
                 start_of_slack.append(section.pointerto_raw_data + section.virtual_size)
 
         if save_to_json_path is not None:
-            with open(save_to_json_path, "w") as outfile:
+            with open(save_to_json_path, "w", encoding="utf8") as outfile:
                 json.dump(cleaned_dump, outfile, indent=4, sort_keys=True)
 
         return start_of_slack, size_of_slack
@@ -707,7 +708,8 @@ class MalwareGDTensorFlow(EvasionAttack):
             if self.l_0 < 1:  # l0 is a fraction of the filesize
                 # if its a filepath we need to get the file size
                 if isinstance(datapoint, str):
-                    open_file = open(datapoint, "rb").read()
+                    with open(datapoint, "rb") as file:
+                        open_file = file.read()
                     sample_size = len(open_file)
                 else:
                     if sample_size is None:
