@@ -259,7 +259,7 @@ class MembershipInferenceBlackBox(MembershipInferenceAttack):
                     optimizer.step()
         else:
             y_ready = check_and_transform_label_format(y_new, len(np.unique(y_new)), return_one_hot=False)
-            self.attack_model.fit(np.c_[x_1, x_2], y_ready)  # type: ignore
+            self.attack_model.fit(np.c_[x_1, x_2], y_ready.ravel())  # type: ignore
 
     def infer(self, x: np.ndarray, y: Optional[np.ndarray] = None, **kwargs) -> np.ndarray:
         """
@@ -279,7 +279,7 @@ class MembershipInferenceBlackBox(MembershipInferenceAttack):
             if self.estimator.input_shape[0] != x.shape[1]:
                 raise ValueError("Shape of x does not match input_shape of estimator")
 
-        if "probabilities" in kwargs.keys():
+        if "probabilities" in kwargs:
             probabilities = kwargs.get("probabilities")
         else:
             probabilities = False
@@ -287,6 +287,9 @@ class MembershipInferenceBlackBox(MembershipInferenceAttack):
         if not self._regressor_model:
             y = check_and_transform_label_format(y, len(np.unique(y)), return_one_hot=True)
 
+        if y is None:
+            raise ValueError("None value detected.")
+
         if y.shape[0] != x.shape[0]:  # pragma: no cover
             raise ValueError("Number of rows in x and y do not match")
 
