@@ -30,10 +30,9 @@ from io import BytesIO
 from typing import Optional, Tuple
 
 import numpy as np
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 from art.defences.preprocessor.preprocessor import Preprocessor
-from art.utils import Deprecated, deprecated_keyword_arg
 
 logger = logging.getLogger(__name__)
 
@@ -43,13 +42,11 @@ class Mp3Compression(Preprocessor):
     Implement the MP3 compression defense approach.
     """
 
-    params = ["channel_index", "channels_first", "sample_rate", "verbose"]
+    params = ["channels_first", "sample_rate", "verbose"]
 
-    @deprecated_keyword_arg("channel_index", end_version="1.6.0", replaced_by="channels_first")
     def __init__(
         self,
         sample_rate: int,
-        channel_index=Deprecated,
         channels_first: bool = False,
         apply_fit: bool = False,
         apply_predict: bool = True,
@@ -59,23 +56,12 @@ class Mp3Compression(Preprocessor):
         Create an instance of MP3 compression.
 
         :param sample_rate: Specifies the sampling rate of sample.
-        :param channel_index: Index of the axis containing the audio channels.
-        :type channel_index: `int`
         :param channels_first: Set channels first or last.
         :param apply_fit: True if applied during fitting/training.
         :param apply_predict: True if applied during predicting.
         :param verbose: Show progress bars.
         """
-        # Remove in 1.6.0
-        if channel_index == 3:
-            channels_first = False
-        elif channel_index == 1:
-            channels_first = True
-        elif channel_index is not Deprecated:
-            raise ValueError("Not a proper channel_index. Use channels_first.")
-
         super().__init__(is_fitted=True, apply_fit=apply_fit, apply_predict=apply_predict)
-        self.channel_index = channel_index
         self.channels_first = channels_first
         self.sample_rate = sample_rate
         self.verbose = verbose
@@ -86,7 +72,7 @@ class Mp3Compression(Preprocessor):
         Apply MP3 compression to sample `x`.
 
         :param x: Sample to compress with shape `(batch_size, length, channel)` or an array of sample arrays with shape
-                  (length,) or (length, channel). `x` values are recommended to be of type `np.int16`.
+                  (length,) or (length, channel).
         :param y: Labels of the sample `x`. This function does not affect them in any way.
         :return: Compressed sample.
         """
@@ -95,21 +81,15 @@ class Mp3Compression(Preprocessor):
             """
             Apply MP3 compression to audio input of shape (samples, channel).
             """
-            # WARNING: Writing and reading MP3 from byte stream causes pydub to extend the original
-            # length. Writing and reading MP3 from local file system works without problems. It is
-            # easy to move from using BytesIO to local read/writes with the following:
-            # import os
-            # from art import config
-            # tmp_wav = os.path.join(config.ART_DATA_PATH, "tmp.wav")
-            # tmp_mp3 = os.path.join(config.ART_DATA_PATH, "tmp.mp3")
             from pydub import AudioSegment
             from scipy.io.wavfile import write
 
+            x_dtype = x.dtype
             normalized = bool(x.min() >= -1.0 and x.max() <= 1.0)
-            if x.dtype != np.int16 and not normalized:
+            if x_dtype != np.int16 and not normalized:
                 # input is not of type np.int16 and seems to be unnormalized. Therefore casting to np.int16.
                 x = x.astype(np.int16)
-            elif x.dtype != np.int16 and normalized:
+            elif x_dtype != np.int16 and normalized:
                 # x is not of type np.int16 and seems to be normalized. Therefore undoing normalization and
                 # casting to np.int16.
                 x = (x * 2 ** 15).astype(np.int16)
@@ -121,25 +101,31 @@ class Mp3Compression(Preprocessor):
             tmp_wav.close()
             tmp_mp3.close()
             x_mp3 = np.array(audio_segment.get_array_of_samples()).reshape((-1, audio_segment.channels))
-            # WARNING: Due to above problem, we need to manually resize x_mp3 to original length.
-            x_mp3 = x_mp3[: x.shape[0]]
+
+            # WARNING: Sometimes we *still* need to manually resize x_mp3 to original length.
+            # This should not be the case, e.g. see https://github.com/jiaaro/pydub/issues/474
+            if x.shape[0] != x_mp3.shape[0]:  # pragma: no cover
+                logger.warning(
+                    "Lengths original input and compressed output don't match. Truncating compressed result."
+                )
+                x_mp3 = x_mp3[: x.shape[0]]
 
             if normalized:
                 # x was normalized. Therefore normalizing x_mp3.
                 x_mp3 = x_mp3 * 2 ** -15
-            return x_mp3
+            return x_mp3.astype(x_dtype)
 
-        if x.dtype != np.object and x.ndim != 3:
+        if x.dtype != object and x.ndim != 3:
             raise ValueError("Mp3 compression can only be applied to temporal data across at least one channel.")
 
-        if x.dtype != np.object and self.channels_first:
+        if x.dtype != object and self.channels_first:
             x = np.swapaxes(x, 1, 2)
 
         # apply mp3 compression per audio item
         x_mp3 = x.copy()
         for i, x_i in enumerate(tqdm(x, desc="MP3 compression", disable=not self.verbose)):
             x_i_ndim_0 = x_i.ndim
-            if x.dtype == np.object:
+            if x.dtype == object:
                 if x_i.ndim == 1:
                     x_i = np.expand_dims(x_i, axis=1)
 
@@ -148,7 +134,7 @@ class Mp3Compression(Preprocessor):
 
             x_i = wav_to_mp3(x_i, self.sample_rate)
 
-            if x.dtype == np.object:
+            if x.dtype == object:
                 if x_i_ndim_0 == 2 and self.channels_first:
                     x_i = np.swapaxes(x_i, 0, 1)
 
@@ -157,13 +143,13 @@ class Mp3Compression(Preprocessor):
 
             x_mp3[i] = x_i
 
-        if x.dtype != np.object and self.channels_first:
+        if x.dtype != object and self.channels_first:
             x_mp3 = np.swapaxes(x_mp3, 1, 2)
 
         return x_mp3, y
 
     def _check_params(self) -> None:
-        if not (isinstance(self.sample_rate, (int, np.int)) and self.sample_rate > 0):
+        if not (isinstance(self.sample_rate, int) and self.sample_rate > 0):
             raise ValueError("Sample rate be must a positive integer.")
 
         if not isinstance(self.verbose, bool):
