@@ -81,12 +81,18 @@ class AdversarialTrainerMadryPGD(Trainer):
 
         # Setting up adversary and perform adversarial training:
         self.attack = ProjectedGradientDescent(
-            classifier, eps=eps, eps_step=eps_step, max_iter=max_iter, num_random_init=num_random_init,
+            classifier,
+            eps=eps,
+            eps_step=eps_step,
+            max_iter=max_iter,
+            num_random_init=num_random_init,
         )
 
         self.trainer = AdversarialTrainer(classifier, self.attack, ratio=1.0)  # type: ignore
 
-    def fit(self, x: np.ndarray, y: np.ndarray, validation_data: Optional[np.ndarray] = None, **kwargs) -> None:
+    def fit(  # pylint: disable=W0221
+        self, x: np.ndarray, y: np.ndarray, validation_data: Optional[np.ndarray] = None, **kwargs
+    ) -> None:
         """
         Train a model adversarially. See class documentation for more information on the exact procedure.
 
