@@ -2,4 +2,6 @@
 Module implementing transformer-based defences against adversarial attacks.
 """
 from art.defences.transformer.transformer import Transformer
-from art.defences.transformer.defensive_distillation import DefensiveDistillation
+
+from art.defences.transformer import evasion
+from art.defences.transformer import poisoning
