@@ -97,6 +97,8 @@ if TYPE_CHECKING:
     )
     from art.estimators.classification.tensorflow import TensorFlowClassifier, TensorFlowV2Classifier
     from art.estimators.classification.xgboost import XGBoostClassifier
+    from art.estimators.generation import TensorFlowGenerator
+    from art.estimators.generation.tensorflow import TensorFlow2Generator
     from art.estimators.object_detection.object_detector import ObjectDetector
     from art.estimators.object_detection.python_object_detector import PyTorchObjectDetector
     from art.estimators.object_detection.pytorch_faster_rcnn import PyTorchFasterRCNN
@@ -184,6 +186,8 @@ if TYPE_CHECKING:
         CLASSIFIER_NEURALNETWORK_TYPE,
     ]
 
+    GENERATOR_TYPE = Union[TensorFlowGenerator, TensorFlow2Generator]  # pylint: disable=C0103
+
     REGRESSOR_TYPE = Union[ScikitlearnRegressor, ScikitlearnDecisionTreeRegressor]  # pylint: disable=C0103
 
     OBJECT_DETECTOR_TYPE = Union[  # pylint: disable=C0103
@@ -334,13 +338,145 @@ def deprecated_keyword_arg(identifier: str, end_version: str, *, reason: str = "
 # ----------------------------------------------------------------------------------------------------- MATH OPERATIONS
 
 
+def projection_l1_1(values: np.ndarray, eps: Union[int, float, np.ndarray]) -> np.ndarray:
+    """
+    This function computes the orthogonal projections of a batch of points on L1-balls of given radii
+    The batch size is  m = values.shape[0].  The points are flattened to dimension
+    n = np.prod(value.shape[1:]).  This is required to facilitate sorting.
+
+    If a[0] <= ... <= a[n-1], then the projection can be characterized using the largest  j  such that
+    a[j+1] +...+ a[n-1] - a[j]*(n-j-1) >= eps. The  ith  coordinate of projection is equal to  0
+    if i=0,...,j.
+
+    :param values:  A batch of  m  points, each an ndarray
+    :param eps:  The radii of the respective L1-balls
+    :return: projections
+    """
+    # pylint: disable=C0103
+
+    shp = values.shape
+    a = values.copy()
+    n = np.prod(a.shape[1:])
+    m = a.shape[0]
+    a = a.reshape((m, n))
+    sgns = np.sign(a)
+    a = np.abs(a)
+
+    a_argsort = a.argsort(axis=1)
+    a_sorted = np.zeros((m, n))
+    for i in range(m):
+        a_sorted[i, :] = a[i, a_argsort[i, :]]
+    a_argsort_inv = a.argsort(axis=1).argsort(axis=1)
+    mat = np.zeros((m, 2))
+
+    #   if  a_sorted[i, n-1]  >= a_sorted[i, n-2] + eps,  then the projection is  [0,...,0,eps]
+    done = False
+    active = np.array([1] * m)
+    after_vec = np.zeros((m, n))
+    proj = a_sorted.copy()
+    j = n - 2
+    while j >= 0:
+        mat[:, 0] = mat[:, 0] + a_sorted[:, j + 1]  # =  sum(a_sorted[: i] :  i = j + 1,...,n-1
+        mat[:, 1] = a_sorted[:, j] * (n - j - 1) + eps
+        #  Find the max in each problem  max{ sum{a_sorted[:, i] : i=j+1,..,n-1} , a_sorted[:, j] * (n-j-1) + eps }
+        row_maxes = np.max(mat, axis=1)
+        #  Set to  1  if  max >  a_sorted[:, j] * (n-j-1) + eps  >  sum ;  otherwise, set to  0
+        ind_set = np.sign(np.sign(row_maxes - mat[:, 0]))
+        #  ind_set = ind_set.reshape((m, 1))
+        #   Multiplier for activation
+        act_multiplier = (1 - ind_set) * active
+        act_multiplier = np.transpose([np.transpose(act_multiplier)] * n)
+        #  if done, the projection is supported by the current indices  j+1,..,n-1   and the amount by which each
+        #  has to be reduced is  delta
+        delta = (mat[:, 0] - eps) / (n - j - 1)
+        #    The vector of reductions
+        delta_vec = np.array([delta] * (n - j - 1))
+        delta_vec = np.transpose(delta_vec)
+        #   The sub-vectors:  a_sorted[:, (j+1):]
+        a_sub = a_sorted[:, (j + 1) :]
+        #   After reduction by delta_vec
+        a_after = a_sub - delta_vec
+        after_vec[:, (j + 1) :] = a_after
+        proj = (act_multiplier * after_vec) + ((1 - act_multiplier) * proj)
+        active = active * ind_set
+        if sum(active) == 0:
+            done = True
+            break
+        j -= 1
+    if not done:
+        proj = active * a_sorted + (1 - active) * proj
+
+    for i in range(m):
+        proj[i, :] = proj[i, a_argsort_inv[i, :]]
+
+    proj = sgns * proj
+    proj = proj.reshape(shp)
+
+    return proj
+
+
+def projection_l1_2(values: np.ndarray, eps: Union[int, float, np.ndarray]) -> np.ndarray:
+    """
+    This function computes the orthogonal projections of a batch of points on L1-balls of given radii
+    The batch size is  m = values.shape[0].  The points are flattened to dimension
+    n = np.prod(value.shape[1:]).  This is required to facilitate sorting.
+
+    Starting from a vector  a = (a1,...,an)  such that  a1 >= ... >= an >= 0,  a1 + ... + an > 1,
+    we first move to  a' = a - (t,...,t)  such that either a1 + ... + an >= 1 ,  an >= 0,
+    and  min( a1 + ... + an  - nt - 1, an -t ) = 0.  This means  t = min( (a1 + ... + an - 1)/n, an).
+    If  t = (a1 + ... + an - 1)/n , then  a' is the desired projection.  Otherwise, the problem is reduced to
+    finding the projection of  (a1 - t, ... , a{n-1} - t ).
+
+    :param values:  A batch of  m  points, each an ndarray
+    :param eps:  The radii of the respective L1-balls
+    :return: projections
+    """
+    # pylint: disable=C0103
+    shp = values.shape
+    a = values.copy()
+    n = np.prod(a.shape[1:])
+    m = a.shape[0]
+    a = a.reshape((m, n))
+    sgns = np.sign(a)
+    a = np.abs(a)
+    a_argsort = a.argsort(axis=1)
+    a_sorted = np.zeros((m, n))
+    for i in range(m):
+        a_sorted[i, :] = a[i, a_argsort[i, :]]
+
+    a_argsort_inv = a.argsort(axis=1).argsort(axis=1)
+    row_sums = np.sum(a, axis=1)
+    mat = np.zeros((m, 2))
+    mat0 = np.zeros((m, 2))
+    a_var = a_sorted.copy()
+    for j in range(n):
+        mat[:, 0] = (row_sums - eps) / (n - j)
+        mat[:, 1] = a_var[:, j]
+        mat0[:, 1] = np.min(mat, axis=1)
+        min_t = np.max(mat0, axis=1)
+        if np.max(min_t) < 1e-8:
+            break
+        row_sums = row_sums - a_var[:, j] * (n - j)
+        a_var[:, (j + 1) :] = a_var[:, (j + 1) :] - np.matmul(min_t.reshape((m, 1)), np.ones((1, n - j - 1)))
+        a_var[:, j] = a_var[:, j] - min_t
+    proj = np.zeros((m, n))
+    for i in range(m):
+        proj[i, :] = a_var[i, a_argsort_inv[i, :]]
+
+    proj = sgns * proj
+    proj = proj.reshape(shp)
+    return proj
+
+
 def projection(values: np.ndarray, eps: Union[int, float, np.ndarray], norm_p: Union[int, float, str]) -> np.ndarray:
     """
     Project `values` on the L_p norm ball of size `eps`.
 
     :param values: Array of perturbations to clip.
     :param eps: Maximum norm allowed.
-    :param norm_p: L_p norm to use for clipping. Only 1, 2, `np.Inf` and "inf" supported for now.
+    :param norm_p: L_p norm to use for clipping.
+            Only 1, 2 , `np.Inf` 1.1 and 1.2 supported for now.
+            1.1 and 1.2 compute orthogonal projections on l1-ball, using two different algorithms
     :return: Values of `values` after projection.
     """
     # Pick a small scalar to avoid division by 0
@@ -363,11 +499,15 @@ def projection(values: np.ndarray, eps: Union[int, float, np.ndarray], norm_p: U
             np.minimum(1.0, eps / (np.linalg.norm(values_tmp, axis=1, ord=1) + tol)),
             axis=1,
         )
+    elif norm_p == 1.1:
+        values_tmp = projection_l1_1(values_tmp, eps)
+    elif norm_p == 1.2:
+        values_tmp = projection_l1_2(values_tmp, eps)
 
     elif norm_p in [np.inf, "inf"]:
         if isinstance(eps, np.ndarray):
             eps = eps * np.ones_like(values)
-            eps = eps.reshape([eps.shape[0], -1])
+            eps = eps.reshape([eps.shape[0], -1])  # type: ignore
 
         values_tmp = np.sign(values_tmp) * np.minimum(abs(values_tmp), eps)
 
@@ -428,7 +568,7 @@ def random_sphere(
         res = np.random.uniform(-radius, radius, (nb_points, nb_dims))
 
     else:
-        raise NotImplementedError("Norm {} not supported".format(norm))
+        raise NotImplementedError(f"Norm {norm} not supported")
 
     return res
 
@@ -537,35 +677,35 @@ def check_and_transform_label_format(
     :param return_one_hot: True if returning one-hot encoded labels, False if returning index labels.
     :return: Labels with shape `(nb_samples, nb_classes)` (one-hot) or `(nb_samples,)` (index).
     """
-    if labels is not None:
-        if len(labels.shape) == 2 and labels.shape[1] > 1:  # multi-class, one-hot encoded
-            if not return_one_hot:
-                labels = np.argmax(labels, axis=1)
-                labels = np.expand_dims(labels, axis=1)
-        elif (
-            len(labels.shape) == 2 and labels.shape[1] == 1 and nb_classes is not None and nb_classes > 2
-        ):  # multi-class, index labels
-            if return_one_hot:
-                labels = to_categorical(labels, nb_classes)
-            else:
-                labels = np.expand_dims(labels, axis=1)
-        elif (
-            len(labels.shape) == 2 and labels.shape[1] == 1 and nb_classes is not None and nb_classes == 2
-        ):  # binary, index labels
-            if return_one_hot:
-                labels = to_categorical(labels, nb_classes)
-        elif len(labels.shape) == 1:  # index labels
-            if return_one_hot:
-                labels = to_categorical(labels, nb_classes)
-            else:
-                labels = np.expand_dims(labels, axis=1)
-        else:
-            raise ValueError(
-                "Shape of labels not recognised."
-                "Please provide labels in shape (nb_samples,) or (nb_samples, nb_classes)"
-            )
+    labels_return = labels
 
-    return labels
+    if len(labels.shape) == 2 and labels.shape[1] > 1:  # multi-class, one-hot encoded
+        if not return_one_hot:
+            labels_return = np.argmax(labels, axis=1)
+            labels_return = np.expand_dims(labels_return, axis=1)
+    elif (
+        len(labels.shape) == 2 and labels.shape[1] == 1 and nb_classes is not None and nb_classes > 2
+    ):  # multi-class, index labels
+        if return_one_hot:
+            labels_return = to_categorical(labels, nb_classes)
+        else:
+            labels_return = np.expand_dims(labels, axis=1)
+    elif (
+        len(labels.shape) == 2 and labels.shape[1] == 1 and nb_classes is not None and nb_classes == 2
+    ):  # binary, index labels
+        if return_one_hot:
+            labels_return = to_categorical(labels, nb_classes)
+    elif len(labels.shape) == 1:  # index labels
+        if return_one_hot:
+            labels_return = to_categorical(labels, nb_classes)
+        else:
+            labels_return = np.expand_dims(labels, axis=1)
+    else:
+        raise ValueError(
+            "Shape of labels not recognised." "Please provide labels in shape (nb_samples,) or (nb_samples, nb_classes)"
+        )
+
+    return labels_return
 
 
 def random_targets(labels: np.ndarray, nb_classes: int) -> np.ndarray:
@@ -649,6 +789,56 @@ def get_labels_np_array(preds: np.ndarray) -> np.ndarray:
     return y
 
 
+def get_feature_values(x: np.ndarray, single_index_feature: bool) -> list:
+    """
+    Returns a list of unique values of a given feature.
+
+    :param x: The feature column(s).
+    :param single_index_feature: Bool representing whether this is a single-column or multiple-column feature (for
+                                 example 1-hot encoded and then scaled).
+    :return: For a single-column feature, a simple list containing all possible values, in increasing order.
+             For a multi-column feature, a list of lists, where each internal list represents a column and the values
+             represent the possible values for that column (in increasing order).
+    """
+    values = None
+    if single_index_feature:
+        values = np.unique(x).tolist()
+    else:
+        for column in x.T:
+            column_values = np.unique(column)
+            if values is None:
+                values = column_values
+            else:
+                values = np.vstack((values, column_values))
+        if values is not None:
+            values = values.tolist()
+    return values
+
+
+def get_feature_index(feature: Union[int, slice]) -> Union[int, slice]:
+    """
+    Returns a modified feature index: in case of a slice of size 1, returns the corresponding integer. Otherwise,
+    returns the same value (integer or slice) as passed.
+
+    :param feature: The index or slice representing a feature to attack
+    :return: An integer representing a single column index or a slice representing a multi-column index
+    """
+    if isinstance(feature, int):
+        return feature
+
+    start = feature.start
+    stop = feature.stop
+    step = feature.step
+    if start is None:
+        start = 0
+    if step is None:
+        step = 1
+    if feature.stop is not None and ((stop - start) // step) == 1:
+        return start
+
+    return feature
+
+
 def compute_success_array(
     classifier: "CLASSIFIER_TYPE",
     x_clean: np.ndarray,
@@ -711,7 +901,7 @@ def compute_success(
     return np.sum(attack_success) / x_adv.shape[0]
 
 
-def compute_accuracy(preds: np.ndarray, labels: np.ndarray, abstain: bool = True) -> Tuple[np.ndarray, int]:
+def compute_accuracy(preds: np.ndarray, labels: np.ndarray, abstain: bool = True) -> Tuple[float, float]:
     """
     Compute the accuracy rate and coverage rate of predictions
     In the case where predictions are abstained, those samples are ignored.
@@ -899,7 +1089,7 @@ def load_iris(raw: bool = False, test_set: float = 0.3) -> DATASET_TYPE:
         data /= np.amax(data)
     labels = to_categorical(iris.target, 3)
 
-    min_, max_ = np.amin(data), np.amax(data)
+    min_, max_ = float(np.amin(data)), float(np.amax(data))
 
     # Split training and test sets
     split_index = int((1 - test_set) * len(data) / 3)
@@ -940,7 +1130,7 @@ def load_iris(raw: bool = False, test_set: float = 0.3) -> DATASET_TYPE:
     random_indices = np.random.permutation(len(y_train))
     x_train, y_train = x_train[random_indices].astype(np.float32), y_train[random_indices]
 
-    return (x_train, y_train), (x_test, y_test), min_, max_
+    return (x_train, y_train), (x_test, y_test), min_, max_  # type: ignore
 
 
 def load_diabetes(raw: bool = False, test_set: float = 0.3) -> DATASET_TYPE:
@@ -1025,7 +1215,7 @@ def load_nursery(
             return 2
         if value == "spec_prior":
             return 3
-        raise Exception("Bad label value: %s" % value)
+        raise Exception(f"Bad label value: {value}")
 
     data["label"] = data["label"].apply(modify_label)
     data["children"] = data["children"].apply(lambda x: 4 if x == "more" else x)
@@ -1109,7 +1299,7 @@ def load_dataset(
     if "diabetes" in name:
         return load_diabetes()
 
-    raise NotImplementedError("There is no loader for dataset '{}'.".format(name))
+    raise NotImplementedError(f"There is no loader for dataset '{name}'.")
 
 
 def _extract(full_path: str, path: str) -> bool:
@@ -1122,7 +1312,7 @@ def _extract(full_path: str, path: str) -> bool:
             archive = tarfile.open(full_path, "r:gz")
     elif full_path.endswith("zip"):  # pragma: no cover
         if zipfile.is_zipfile(full_path):
-            archive = zipfile.ZipFile(full_path)
+            archive = zipfile.ZipFile(full_path)  # pylint: disable=R1732
         else:
             return False
     else:
@@ -1209,7 +1399,7 @@ def get_file(filename: str, url: str, path: Optional[str] = None, extract: bool
             except HTTPError as exception:  # pragma: no cover
                 raise Exception(error_msg.format(url, exception.code, exception.msg)) from HTTPError  # type: ignore
             except URLError as exception:  # pragma: no cover
-                raise Exception(error_msg.format(url, exception.errno, exception.reason)) from HTTPError
+                raise Exception(error_msg.format(url, exception.errno, exception.reason)) from HTTPError  # type: ignore
         except (Exception, KeyboardInterrupt):  # pragma: no cover
             if os.path.exists(full_path):
                 os.remove(full_path)
@@ -1294,7 +1484,7 @@ def segment_by_class(data: Union[np.ndarray, List[int]], classes: np.ndarray, nu
     by_class: List[List[int]] = [[] for _ in range(num_classes)]
     for indx, feature in enumerate(classes):
         if len(classes.shape) == 2 and classes.shape[1] > 1:
-            assigned = np.argmax(feature)
+            assigned = int(np.argmax(feature))
         else:
             assigned = int(feature)
         by_class[assigned].append(data[indx])
@@ -1346,7 +1536,7 @@ def performance_diff(
     if callable(perf_function):
         return perf_function(test_labels, model1_labels, **kwargs) - perf_function(test_labels, model2_labels, **kwargs)
 
-    raise ValueError("Performance function '{}' not supported".format(str(perf_function)))
+    raise ValueError(f"Performance function '{perf_function}' not supported")
 
 
 def is_probability(vector: np.ndarray) -> bool:
