@@ -40,7 +40,8 @@ logger = logging.getLogger(__name__)
 
 
 @pytest.mark.skip_framework("dl_frameworks")
-def test_black_box(art_warning, decision_tree_estimator, get_iris_dataset):
+@pytest.mark.parametrize("model_type", ["nn", "rf"])
+def test_black_box(art_warning, decision_tree_estimator, get_iris_dataset, model_type):
     try:
         attack_feature = 2  # petal length
 
@@ -70,27 +71,181 @@ def test_black_box(art_warning, decision_tree_estimator, get_iris_dataset):
 
         classifier = decision_tree_estimator()
 
-        attack = AttributeInferenceBlackBox(classifier, attack_feature=attack_feature)
+        attack = AttributeInferenceBlackBox(classifier, attack_feature=attack_feature, attack_model_type=model_type)
         # get original model's predictions
         x_train_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_train_iris)]).reshape(-1, 1)
         x_test_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_test_iris)]).reshape(-1, 1)
         # train attack model
         attack.fit(x_train)
         # infer attacked feature
-        inferred_train = attack.infer(x_train_for_attack, x_train_predictions, values=values)
-        inferred_test = attack.infer(x_test_for_attack, x_test_predictions, values=values)
+        inferred_train = attack.infer(x_train_for_attack, pred=x_train_predictions, values=values)
+        inferred_test = attack.infer(x_test_for_attack, pred=x_test_predictions, values=values)
         # check accuracy
         train_acc = np.sum(inferred_train == x_train_feature.reshape(1, -1)) / len(inferred_train)
         test_acc = np.sum(inferred_test == x_test_feature.reshape(1, -1)) / len(inferred_test)
-        assert train_acc == pytest.approx(0.8285, abs=0.12)
-        assert test_acc == pytest.approx(0.8888, abs=0.12)
+        assert pytest.approx(0.8285, abs=0.2) == train_acc
+        assert pytest.approx(0.8888, abs=0.18) == test_acc
 
     except ARTTestException as e:
         art_warning(e)
 
 
 @pytest.mark.skip_framework("dl_frameworks")
-def test_black_box_regressor(art_warning, get_diabetes_dataset):
+@pytest.mark.parametrize("model_type", ["nn", "rf"])
+def test_black_box_slice(art_warning, decision_tree_estimator, get_iris_dataset, model_type):
+    try:
+        attack_feature = 2  # petal length
+
+        # need to transform attacked feature into categorical
+        def transform_feature(x):
+            x[x > 0.5] = 2.0
+            x[(x > 0.2) & (x <= 0.5)] = 1.0
+            x[x <= 0.2] = 0.0
+
+        values = [0.0, 1.0, 2.0]
+
+        (x_train_iris, y_train_iris), (x_test_iris, y_test_iris) = get_iris_dataset
+        # training data without attacked feature
+        x_train_for_attack = np.delete(x_train_iris, attack_feature, 1)
+        # only attacked feature
+        x_train_feature = x_train_iris[:, attack_feature].copy().reshape(-1, 1)
+        transform_feature(x_train_feature)
+        # training data with attacked feature (after transformation)
+        x_train = np.concatenate((x_train_for_attack[:, :attack_feature], x_train_feature), axis=1)
+        x_train = np.concatenate((x_train, x_train_for_attack[:, attack_feature:]), axis=1)
+
+        # test data without attacked feature
+        x_test_for_attack = np.delete(x_test_iris, attack_feature, 1)
+        # only attacked feature
+        x_test_feature = x_test_iris[:, attack_feature].copy().reshape(-1, 1)
+        transform_feature(x_test_feature)
+
+        classifier = decision_tree_estimator()
+
+        attack = AttributeInferenceBlackBox(
+            classifier, attack_feature=slice(attack_feature, attack_feature + 1), attack_model_type=model_type
+        )
+        # get original model's predictions
+        x_train_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_train_iris)]).reshape(-1, 1)
+        x_test_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_test_iris)]).reshape(-1, 1)
+        # train attack model
+        attack.fit(x_train)
+        # infer attacked feature
+        inferred_train = attack.infer(x_train_for_attack, pred=x_train_predictions, values=values)
+        inferred_test = attack.infer(x_test_for_attack, pred=x_test_predictions, values=values)
+        # check accuracy
+        train_acc = np.sum(inferred_train == x_train_feature.reshape(1, -1)) / len(inferred_train)
+        test_acc = np.sum(inferred_test == x_test_feature.reshape(1, -1)) / len(inferred_test)
+        assert pytest.approx(0.8285, abs=0.12) == train_acc
+        assert pytest.approx(0.8888, abs=0.18) == test_acc
+
+    except ARTTestException as e:
+        art_warning(e)
+
+
+@pytest.mark.skip_framework("dl_frameworks")
+@pytest.mark.parametrize("model_type", ["nn", "rf"])
+def test_black_box_with_label(art_warning, decision_tree_estimator, get_iris_dataset, model_type):
+    try:
+        attack_feature = 2  # petal length
+
+        # need to transform attacked feature into categorical
+        def transform_feature(x):
+            x[x > 0.5] = 2.0
+            x[(x > 0.2) & (x <= 0.5)] = 1.0
+            x[x <= 0.2] = 0.0
+
+        values = [0.0, 1.0, 2.0]
+
+        (x_train_iris, y_train_iris), (x_test_iris, y_test_iris) = get_iris_dataset
+        # training data without attacked feature
+        x_train_for_attack = np.delete(x_train_iris, attack_feature, 1)
+        # only attacked feature
+        x_train_feature = x_train_iris[:, attack_feature].copy().reshape(-1, 1)
+        transform_feature(x_train_feature)
+        # training data with attacked feature (after transformation)
+        x_train = np.concatenate((x_train_for_attack[:, :attack_feature], x_train_feature), axis=1)
+        x_train = np.concatenate((x_train, x_train_for_attack[:, attack_feature:]), axis=1)
+
+        # test data without attacked feature
+        x_test_for_attack = np.delete(x_test_iris, attack_feature, 1)
+        # only attacked feature
+        x_test_feature = x_test_iris[:, attack_feature].copy().reshape(-1, 1)
+        transform_feature(x_test_feature)
+
+        classifier = decision_tree_estimator()
+
+        attack = AttributeInferenceBlackBox(classifier, attack_feature=attack_feature, attack_model_type=model_type)
+        # get original model's predictions
+        x_train_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_train_iris)]).reshape(-1, 1)
+        x_test_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_test_iris)]).reshape(-1, 1)
+        # train attack model
+        attack.fit(x_train, y=y_train_iris)
+        # infer attacked feature
+        inferred_train = attack.infer(x_train_for_attack, y=y_train_iris, pred=x_train_predictions, values=values)
+        inferred_test = attack.infer(x_test_for_attack, y=y_test_iris, pred=x_test_predictions, values=values)
+        # check accuracy
+        train_acc = np.sum(inferred_train == x_train_feature.reshape(1, -1)) / len(inferred_train)
+        test_acc = np.sum(inferred_test == x_test_feature.reshape(1, -1)) / len(inferred_test)
+        assert pytest.approx(0.8285, abs=0.12) == train_acc
+        assert pytest.approx(0.8888, abs=0.18) == test_acc
+
+    except ARTTestException as e:
+        art_warning(e)
+
+
+@pytest.mark.skip_framework("dl_frameworks")
+@pytest.mark.parametrize("model_type", ["nn", "rf"])
+def test_black_box_no_values(art_warning, decision_tree_estimator, get_iris_dataset, model_type):
+    try:
+        attack_feature = 2  # petal length
+
+        # need to transform attacked feature into categorical
+        def transform_feature(x):
+            x[x > 0.5] = 2.0
+            x[(x > 0.2) & (x <= 0.5)] = 1.0
+            x[x <= 0.2] = 0.0
+
+        (x_train_iris, y_train_iris), (x_test_iris, y_test_iris) = get_iris_dataset
+        # training data without attacked feature
+        x_train_for_attack = np.delete(x_train_iris, attack_feature, 1)
+        # only attacked feature
+        x_train_feature = x_train_iris[:, attack_feature].copy().reshape(-1, 1)
+        transform_feature(x_train_feature)
+        # training data with attacked feature (after transformation)
+        x_train = np.concatenate((x_train_for_attack[:, :attack_feature], x_train_feature), axis=1)
+        x_train = np.concatenate((x_train, x_train_for_attack[:, attack_feature:]), axis=1)
+
+        # test data without attacked feature
+        x_test_for_attack = np.delete(x_test_iris, attack_feature, 1)
+        # only attacked feature
+        x_test_feature = x_test_iris[:, attack_feature].copy().reshape(-1, 1)
+        transform_feature(x_test_feature)
+
+        classifier = decision_tree_estimator()
+
+        attack = AttributeInferenceBlackBox(classifier, attack_feature=attack_feature, attack_model_type=model_type)
+        # get original model's predictions
+        x_train_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_train_iris)]).reshape(-1, 1)
+        x_test_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_test_iris)]).reshape(-1, 1)
+        # train attack model
+        attack.fit(x_train)
+        # infer attacked feature
+        inferred_train = attack.infer(x_train_for_attack, pred=x_train_predictions)
+        inferred_test = attack.infer(x_test_for_attack, pred=x_test_predictions)
+        # check accuracy
+        train_acc = np.sum(inferred_train == x_train_feature.reshape(1, -1)) / len(inferred_train)
+        test_acc = np.sum(inferred_test == x_test_feature.reshape(1, -1)) / len(inferred_test)
+        assert pytest.approx(0.8285, abs=0.12) == train_acc
+        assert pytest.approx(0.8888, abs=0.18) == test_acc
+
+    except ARTTestException as e:
+        art_warning(e)
+
+
+@pytest.mark.skip_framework("dl_frameworks")
+@pytest.mark.parametrize("model_type", ["nn", "rf"])
+def test_black_box_regressor(art_warning, get_diabetes_dataset, model_type):
     try:
         attack_feature = 0  # age
 
@@ -137,21 +292,23 @@ def test_black_box_regressor(art_warning, get_diabetes_dataset):
         regr_model.fit(x_train_diabetes, y_train_diabetes)
         regressor = ScikitlearnRegressor(regr_model)
 
-        attack = AttributeInferenceBlackBox(regressor, attack_feature=attack_feature, prediction_normal_factor=1 / 250)
+        attack = AttributeInferenceBlackBox(
+            regressor, attack_feature=attack_feature, prediction_normal_factor=1 / 250, attack_model_type=model_type
+        )
         # get original model's predictions
         x_train_predictions = regressor.predict(x_train_diabetes).reshape(-1, 1)
         x_test_predictions = regressor.predict(x_test_diabetes).reshape(-1, 1)
         # train attack model
         attack.fit(x_train)
         # infer attacked feature
-        inferred_train = attack.infer(x_train_for_attack, x_train_predictions, values=values)
-        inferred_test = attack.infer(x_test_for_attack, x_test_predictions, values=values)
+        inferred_train = attack.infer(x_train_for_attack, pred=x_train_predictions, values=values)
+        inferred_test = attack.infer(x_test_for_attack, pred=x_test_predictions, values=values)
         # check accuracy
         train_acc = np.sum(inferred_train == x_train_feature.reshape(1, -1)) / len(inferred_train)
         test_acc = np.sum(inferred_test == x_test_feature.reshape(1, -1)) / len(inferred_test)
 
-        assert train_acc == pytest.approx(0.0258, abs=0.12)
-        assert test_acc == pytest.approx(0.0375, abs=0.12)
+        assert pytest.approx(0.0258, abs=0.12) == train_acc
+        assert pytest.approx(0.0375, abs=0.12) == test_acc
 
     except ARTTestException as e:
         art_warning(e)
@@ -204,8 +361,8 @@ def test_black_box_with_model(art_warning, decision_tree_estimator, get_iris_dat
         # train attack model
         attack.fit(x_train)
         # infer attacked feature
-        inferred_train = attack.infer(x_train_for_attack, x_train_predictions, values=values)
-        inferred_test = attack.infer(x_test_for_attack, x_test_predictions, values=values)
+        inferred_train = attack.infer(x_train_for_attack, pred=x_train_predictions, values=values)
+        inferred_test = attack.infer(x_test_for_attack, pred=x_test_predictions, values=values)
         # check accuracy
         # train_acc
         _ = np.sum(inferred_train == x_train_feature.reshape(1, -1)) / len(inferred_train)
@@ -218,7 +375,8 @@ def test_black_box_with_model(art_warning, decision_tree_estimator, get_iris_dat
 
 
 @pytest.mark.skip_framework("dl_frameworks")
-def test_black_box_one_hot(art_warning, get_iris_dataset):
+@pytest.mark.parametrize("model_type", ["nn", "rf"])
+def test_black_box_one_hot(art_warning, get_iris_dataset, model_type):
     try:
         attack_feature = 2  # petal length
 
@@ -259,27 +417,30 @@ def test_black_box_one_hot(art_warning, get_iris_dataset):
         tree.fit(x_train, y_train)
         classifier = ScikitlearnDecisionTreeClassifier(tree)
 
-        attack = AttributeInferenceBlackBox(classifier, attack_feature=slice(attack_feature, attack_feature + 3))
+        attack = AttributeInferenceBlackBox(
+            classifier, attack_feature=slice(attack_feature, attack_feature + 3), attack_model_type=model_type
+        )
         # get original model's predictions
         x_train_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_train)]).reshape(-1, 1)
         x_test_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_test)]).reshape(-1, 1)
         # train attack model
         attack.fit(x_train)
         # infer attacked feature
-        inferred_train = attack.infer(x_train_for_attack, x_train_predictions)
-        inferred_test = attack.infer(x_test_for_attack, x_test_predictions)
+        inferred_train = attack.infer(x_train_for_attack, pred=x_train_predictions)
+        inferred_test = attack.infer(x_test_for_attack, pred=x_test_predictions)
         # check accuracy
         train_acc = np.sum(np.all(inferred_train == train_one_hot, axis=1)) / len(inferred_train)
         test_acc = np.sum(np.all(inferred_test == test_one_hot, axis=1)) / len(inferred_test)
-        assert pytest.approx(0.8666, abs=0.03) == train_acc
-        assert pytest.approx(0.8888, abs=0.03) == test_acc
+        assert pytest.approx(0.8666, abs=0.12) == train_acc
+        assert pytest.approx(0.8888, abs=0.7) == test_acc
 
     except ARTTestException as e:
         art_warning(e)
 
 
 @pytest.mark.skip_framework("dl_frameworks")
-def test_black_box_one_hot_float(art_warning, get_iris_dataset):
+@pytest.mark.parametrize("model_type", ["nn", "rf"])
+def test_black_box_one_hot_float(art_warning, get_iris_dataset, model_type):
     try:
         attack_feature = 2  # petal length
 
@@ -332,7 +493,7 @@ def test_black_box_one_hot_float(art_warning, get_iris_dataset):
         tree.fit(x_train, y_train)
         classifier = ScikitlearnDecisionTreeClassifier(tree)
 
-        attack = AttributeInferenceBlackBox(classifier, attack_feature=attack_feature)
+        attack = AttributeInferenceBlackBox(classifier, attack_feature=attack_feature, attack_model_type=model_type)
         # get original model's predictions
         x_train_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_train)]).reshape(-1, 1)
         x_test_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_test)]).reshape(-1, 1)
@@ -340,8 +501,8 @@ def test_black_box_one_hot_float(art_warning, get_iris_dataset):
         attack.fit(x_train)
         # infer attacked feature
         values = [[-0.559017, 1.7888544], [-0.47003216, 2.127514], [-1.1774395, 0.84930056]]
-        inferred_train = attack.infer(x_train_for_attack, x_train_predictions, values=values)
-        inferred_test = attack.infer(x_test_for_attack, x_test_predictions, values=values)
+        inferred_train = attack.infer(x_train_for_attack, pred=x_train_predictions, values=values)
+        inferred_test = attack.infer(x_test_for_attack, pred=x_test_predictions, values=values)
         # check accuracy
         train_acc = np.sum(
             np.all(np.around(inferred_train, decimals=3) == np.around(train_one_hot, decimals=3), axis=1)
@@ -349,7 +510,85 @@ def test_black_box_one_hot_float(art_warning, get_iris_dataset):
         test_acc = np.sum(
             np.all(np.around(inferred_test, decimals=3) == np.around(test_one_hot, decimals=3), axis=1)
         ) / len(inferred_test)
-        assert pytest.approx(0.8666, abs=0.1) == train_acc
+        assert pytest.approx(0.8666, abs=0.12) == train_acc
+        assert pytest.approx(0.8666, abs=0.1) == test_acc
+
+    except ARTTestException as e:
+        art_warning(e)
+
+
+@pytest.mark.skip_framework("dl_frameworks")
+@pytest.mark.parametrize("model_type", ["nn", "rf"])
+def test_black_box_one_hot_float_no_values(art_warning, get_iris_dataset, model_type):
+    try:
+        attack_feature = 2  # petal length
+
+        # need to transform attacked feature into categorical
+        def transform_feature(x):
+            x[x > 0.5] = 2
+            x[(x > 0.2) & (x <= 0.5)] = 1
+            x[x <= 0.2] = 0
+
+        (x_train_iris, y_train_iris), (x_test_iris, y_test_iris) = get_iris_dataset
+        # training data without attacked feature
+        x_train_for_attack = np.delete(x_train_iris, attack_feature, 1)
+        # only attacked feature
+        x_train_feature = x_train_iris[:, attack_feature].copy().reshape(-1, 1)
+        transform_feature(x_train_feature)
+        # transform to one-hot encoding
+        num_columns = int(x_train_feature.max()) + 1
+        train_one_hot = np.zeros((x_train_feature.size, num_columns))
+        train_one_hot[np.arange(x_train_feature.size), x_train_feature.reshape(1, -1).astype(int)] = 1
+        # training data with attacked feature (after transformation)
+        x_train = np.concatenate((x_train_for_attack[:, :attack_feature], train_one_hot), axis=1)
+        x_train = np.concatenate((x_train, x_train_for_attack[:, attack_feature:]), axis=1)
+
+        y_train = np.array([np.argmax(y) for y in y_train_iris]).reshape(-1, 1)
+
+        # test data without attacked feature
+        x_test_for_attack = np.delete(x_test_iris, attack_feature, 1)
+        # only attacked feature
+        x_test_feature = x_test_iris[:, attack_feature].copy().reshape(-1, 1)
+        transform_feature(x_test_feature)
+        # transform to one-hot encoding
+        test_one_hot = np.zeros((x_test_feature.size, int(x_test_feature.max()) + 1))
+        test_one_hot[np.arange(x_test_feature.size), x_test_feature.reshape(1, -1).astype(int)] = 1
+        # test data with attacked feature (after transformation)
+        x_test = np.concatenate((x_test_for_attack[:, :attack_feature], test_one_hot), axis=1)
+        x_test = np.concatenate((x_test, x_test_for_attack[:, attack_feature:]), axis=1)
+
+        # scale before training
+        scaler = StandardScaler().fit(x_train)
+        x_test = scaler.transform(x_test).astype(np.float32)
+        x_train = scaler.transform(x_train).astype(np.float32)
+        # derive dataset for attack (after scaling)
+        attack_feature = slice(attack_feature, attack_feature + 3)
+        x_train_for_attack = np.delete(x_train, attack_feature, 1)
+        x_test_for_attack = np.delete(x_test, attack_feature, 1)
+        train_one_hot = x_train[:, attack_feature]
+        test_one_hot = x_test[:, attack_feature]
+
+        tree = DecisionTreeClassifier()
+        tree.fit(x_train, y_train)
+        classifier = ScikitlearnDecisionTreeClassifier(tree)
+
+        attack = AttributeInferenceBlackBox(classifier, attack_feature=attack_feature, attack_model_type=model_type)
+        # get original model's predictions
+        x_train_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_train)]).reshape(-1, 1)
+        x_test_predictions = np.array([np.argmax(arr) for arr in classifier.predict(x_test)]).reshape(-1, 1)
+        # train attack model
+        attack.fit(x_train)
+        # infer attacked feature
+        inferred_train = attack.infer(x_train_for_attack, pred=x_train_predictions)
+        inferred_test = attack.infer(x_test_for_attack, pred=x_test_predictions)
+        # check accuracy
+        train_acc = np.sum(
+            np.all(np.around(inferred_train, decimals=3) == np.around(train_one_hot, decimals=3), axis=1)
+        ) / len(inferred_train)
+        test_acc = np.sum(
+            np.all(np.around(inferred_test, decimals=3) == np.around(test_one_hot, decimals=3), axis=1)
+        ) / len(inferred_test)
+        assert pytest.approx(0.8666, abs=0.12) == train_acc
         assert pytest.approx(0.8666, abs=0.1) == test_acc
 
     except ARTTestException as e:
